(* common.ml *)

open Formule
open Sequent
open State

exception Command_error of string

type arg = Arg_id of id | Arg_form of formule

type command =
	| Load of string
	| Save of string
	| Undo
	| Start of sequent
	| Apply of tactic
	| Focus of int
	| Finish
	| Exit	

let make_tac = function
	| "Assume", [] -> tac_assume
	| "Axiom", [] -> tac_axiom
	| "Aff", [ Arg_id i_a ] -> tac_aff i_a
	| "And_l", [ Arg_id i_ab; Arg_id i_a; Arg_id i_b ] -> tac_and_l i_ab i_a i_b
	| "And_l", [ Arg_id i_ab ] -> tac_and_l i_ab ( i_ab^".1" ) ( i_ab^".2" )
	| "And_r", [] -> tac_and_r
	| "Or_l", [ Arg_id i_ab; Arg_id i_a; Arg_id i_b ] -> tac_or_l i_ab i_a i_b
	| "Or_l", [ Arg_id i_ab ] -> tac_or_l i_ab ( i_ab^".1" ) ( i_ab^".2" )
	| "Or_r1", [] -> tac_or_r1
	| "Or_r2", [] -> tac_or_r2
	| "Imply_l", [ Arg_id i_ab; Arg_id i_b ] -> tac_imply_l i_ab i_b
	| "Imply_l", [ Arg_id i_ab ] -> tac_imply_l i_ab i_ab
	| "Imply_r", [ Arg_id i_a ] -> tac_imply_r i_a
	| "Neg_l", [ Arg_id i ] -> tac_neg_l i
	| "Neg_r", [ Arg_id i ] -> tac_neg_r i
	| "Abs", [ Arg_id i ] -> tac_abs i
	| "Cut", [ Arg_id i; Arg_form f ] -> tac_cut i f
	| "Skip", [] -> tac_skip
	| s, _ -> raise ( Command_error s )
