(* formule.ml *)

type formule = 
	| Bottom
	| Atom of char
	| Or of formule*formule
	| And of formule*formule
	| Imply of formule*formule
	| Not of formule

(* Priorité des opérateurs : Imply = 0, Or = 1, And = 2, Not = 3 *)
let print_formule o f = 
	let lpar prec op_prec = if prec > op_prec then "(" else ""
	and rpar prec op_prec = if prec > op_prec then ")" else "" in
	let rec aux prec o = function
		| Bottom -> Printf.fprintf o "_"
		| Atom c -> Printf.fprintf o "%c" c
		| Or (a, b) ->
			Printf.fprintf o "%s%a OU %a%s" 
			( lpar prec 1 ) ( aux 1 ) a ( aux 1 ) b ( rpar prec 1 )
		| And (a, b) ->
			Printf.fprintf o "%s%a ET %a%s" 
			( lpar prec 2 ) ( aux 2 ) a ( aux 2 ) b ( rpar prec 2 )
		| Imply (a, b) -> 
			Printf.fprintf o "%s%a -> %a%s" (
			 lpar prec 0 ) ( aux 0 ) a ( aux 0 ) b ( rpar prec 0 )
		| Not a -> Printf.fprintf o "!%a" ( aux 3 ) a in
	aux 0 o f

(* affiche une formule pour l'insérer dans un document Latex *)
let latex_print_formule o f = 
	let lpar prec op_prec = if prec > op_prec then "(" else ""
	and rpar prec op_prec = if prec > op_prec then ")" else "" in
	let rec aux prec o = function
		| Bottom -> Printf.fprintf o " \\bot "
		| Atom c -> Printf.fprintf o " %c " c
		| Or (a, b) ->
			Printf.fprintf o "%s%a \\vee %a%s" 
			( lpar prec 1 ) ( aux 1 ) a ( aux 1 ) b ( rpar prec 1 )
		| And (a, b) ->
			Printf.fprintf o "%s%a \\wedge %a%s" 
			( lpar prec 2 ) ( aux 2 ) a ( aux 2 ) b ( rpar prec 2 )
		| Imply (a, b) -> 
			Printf.fprintf o "%s%a \\rightarrow %a%s" 
			( lpar prec 0 ) ( aux 0 ) a ( aux 0 ) b ( rpar prec 0 )
		| Not a -> Printf.fprintf o "\\neg %a" ( aux 3 ) a in
	aux 0 o f
