(* history.ml *)

open State

exception Skip

type t = {
	max_size : int;
	mutable curr_size : int;
	mutable curr_pos : int;
	states : state option array;
	mutable commands : string list
}

let mk i =
	assert ( i > 0 );
	{
		max_size = i;
		curr_size = 0;
		curr_pos = 0;
		states = Array.make i None;
		commands = []
	}
	
(* Formate les commandes à enregistrer et évite
   de sauvegarder les commandes /save et /undo *)
let preproc s =
	if String.length s < 5 then s^"\n"
	else
		let sub = String.sub s 0 5 in
		match sub with
		| "/save"
		| "/undo" -> raise Skip
		| _ -> s^"\n"
	
let add h ( str, s ) =
	try
		let p = preproc str in
		h.commands <- p :: h.commands;
		( if h.curr_size < h.max_size then h.curr_size <- h.curr_size + 1 );
		h.curr_pos <- ( h.curr_pos + 1 ) mod h.max_size;
		h.states.( h.curr_pos ) <- s
	with
	| Skip -> ()

let undo h =
	if h.curr_size < 1 then raise Not_found
	else
		let s = h.states.( h.curr_pos ) in
		begin
		h.curr_size <- h.curr_size - 1;
		h.curr_pos <- ( h.curr_pos - 1 ) mod h.max_size - 1;
		assert ( ( List.length h.commands ) > 0 );
		h.commands <- List.tl h.commands
		end;
		s

let reset h =
	h.curr_size <- 0;
	h.commands <- []

let print o h = 
	List.iter ( output_string o ) ( List.rev h.commands )
