/* parser.mly */

%{
	open Formule
	open Sequent
	open Common
	open State
	
	(* Numérote les hypothèses d'un séquent *)
	let rec label i accu = function
		| t :: q ->
			label ( i + 1 ) ( ( String.make 1 ( char_of_int i ), t ) :: accu ) q
		| [] -> List.rev accu
	
	(* Renvoie une commande (x) et sa position (start,len) dans le lexer *)
	let get_pos x =
		let start = Parsing.symbol_start() in
		let len = Parsing.symbol_end() - start in
		( x, start, len )

%}

%token <int> INT
%token <char> VAR
%token <string> ID FILE TAC
%token EOF LPAR RPAR QUOTE COL SEMI END SEP THS
%token LOAD SAVE UNDO EXIT START FOCUS APPLY QED 
%token TRY WITH SEP REPEAT 
%token AND OR IMPLY NOT


%left IMPLY
%left OR
%left AND
%nonassoc NOT
%right SEP
%right SEMI
%nonassoc WITH
%nonassoc TRY
%nonassoc REPEAT


/* on renvoie une commande et sa position dans le lexer */
%type <( Common.command * int * int )> main
%start main

%%

formule:
| VAR                         { Atom $1 }
| formule AND formule         { And ($1, $3) }
| formule OR formule          { Or ($1, $3) }
| formule IMPLY formule       { Imply ($1, $3) }
| NOT formule                 { Not $2 }
| LPAR formule RPAR           { $2 }

id_f_list:
| ID COL formule               { [ ( $1, $3 ) ] }
| ID COL formule SEP id_f_list { ( $1, $3 ) :: $5 }

f_list:
| formule                     { [ $1 ] }
| formule SEP f_list          { $1 :: $3 }

sequent:
| THS formule                 { ( [], $2 ) }
| id_f_list THS formule       { ( $1, $3 ) }
| f_list THS formule          { ( label ( int_of_char 'a') [] $1, $3 ) }

arg:
| ID                          { Arg_id $1 }
| formule                     { Arg_form $1 }

arg_list:
| arg                         { [ $1 ] }
| arg arg_list                { $1 :: $2 }

tac:
| TAC                         { make_tac( $1, [] ) }
| TAC arg_list                { make_tac( $1, $2 ) }
| LPAR tac RPAR               { $2 }
| TRY tac                     { tac_cond $2 tac_skip }
| TRY tac WITH tac            { tac_cond $2 $4 }
| tac SEMI tac                { tac_comp $1 $3 }
| REPEAT tac						{ tac_repeat $2 }


command:
| LOAD FILE                   { Load $2 }
| SAVE FILE                   { Save $2 }
| UNDO                        { Undo }
| EXIT                        { Exit }
| START sequent               { Start $2 }
| FOCUS INT                   { Focus $2 }
| tac                  			{ Apply $1 }
| APPLY tac                   { Apply $2 }
| QED                         { Finish }

main:
| command END                 { get_pos $1 }

