(* proof.mli *)

open Sequent
open Formule

type proof

exception Invalid_proof of string (* si une tactique élémentaire est cassée *)

val assume : sequent -> proof
		(* T |- A, preuve de T |- A*)

val axiom : sequent -> proof
		(* T,A|-A, preuve de T,A|-A *)

val aff : id -> formule -> proof -> proof
		(* B, preuve de T|-A, preuve de T,B|-A *) 

val and_l : id -> id -> id -> proof -> proof
		(* A^B,A,B,preuve de T,A,B|-C, preuve de T,A^B|-C*)
		
val and_r : proof -> proof -> proof
		(* preuve de T|-A, preuve de T|-B, preuve de T|-A^B*)

val or_l : id -> id -> id -> proof -> proof -> proof
		(* AvB,A,B,preuve de T,A|-C, preuve de T,B|-C , preuve de T,AvB|-C*)
		
val or_r1 : formule -> proof -> proof
		(* B, preuve de T|-A, preuve de T|-AvB *)

val or_r2 : formule -> proof -> proof
		(* A, preuve de T|-B, preuve de T|-AvB *)

val imply_l : id -> id -> proof -> proof -> proof
		(* A -> B, B, preuve de T|-A, preuve de T,B|-C, preuve de T,A->B|-C *)

val imply_r : id -> proof -> proof
		(* A, preuve de T,A|-B , preuve de T|- A->B *)

val neg_l : id -> proof -> proof
		(* !A, preuve de T|-A, preuve de T,!A|- _ *)

val neg_r : id -> proof -> proof
		(* A, preuve de T,A|- _, preuve de T|-!A*)
 

val cut : id -> proof -> proof -> proof
		(*A,preuve de T|-A,preuve de T,A|-B,preuve de T,A|-B*)

val abs : id -> proof -> proof
		(*!A,preuve de T,!A|-_,preuve de T|-A*)

val is_intuit : proof -> bool
		
val print_proof : out_channel -> proof -> unit

val print_premisses : out_channel -> proof -> unit

val print_result : out_channel -> proof -> unit


