(* lexer.mll *)

{ 
  open Parser 
  exception Error of char
}
let num = [ '0'-'9' ]
let lcase = [ 'a'-'z' ]
let ucase = [ 'A'-'Z' ]
let skip = [ ' ' '\r' '\t' ]

let tac = ucase ( lcase | num | '_' )+
let id = lcase ( num | lcase | '.' )*
let file = ( ucase | lcase | num | '_' )+ ".proof"

rule token = parse
	| '\n'           { Lexing.new_line lexbuf; token lexbuf }
	| skip+          { token lexbuf }
	| '#'            { comment lexbuf }
	| "/load"        { LOAD }
	| "/save"        { SAVE }
	| "/undo"        { UNDO }
	| "/exit"        { EXIT }
	| "/start"       { START }
	| "/focus"       { FOCUS }
	| "/apply"       { APPLY }	
	| "/qed"         { QED }
	| "repeat"       { REPEAT }
	| "try"          { TRY }
	| "with"         { WITH }
	| "ET"           { AND }
	| "OU"           { OR }
	| '('            { LPAR }
	| ')'            { RPAR }
	| "->"           { IMPLY }
	| '!'            { NOT }
	| ','            { SEP }
	| ':'            { COL }
	| "|-"           { THS }
	| ";;"           { END }
	| ';'            { SEMI }
	| tac as s       { TAC s }
	| num+ as n      { INT ( int_of_string n ) }
	| ucase as c     { VAR c }
	| file as s      { FILE s }
	| id as s        { ID s }
	| eof            { raise End_of_file }
	| _ as c         { raise ( Error c ) }

and comment = parse
	| '\n' { Lexing.new_line lexbuf; token lexbuf }
	| eof  { raise End_of_file }
	| _    { comment lexbuf }
